@@ -64,7 +64,7 @@ class Loan(DataFineractObject):
         self.in_arrears = attributes.get('inArrears', None)
         self.is_npa = attributes.get('isNPA', None)
 
-    def days_in_arrears(self):
+    def days_in_arrears(self) -> int:
         """Return the number of days in arrears
 
         :return: int
@@ -76,6 +76,203 @@ class Loan(DataFineractObject):
 
         return in_arrears
 
+    def is_closed(self) -> bool:
+        """Check whether a loan is closed or not
+
+        :return: bool
+        """
+        return self.status is not None and self.status.closed
+
+    @staticmethod
+    def template(request_handler, client_id=None, template_type='individual', product_id=None) -> bool:
+        """Get a loan template
+
+        :param template_type:
+        :param product_id:
+        :param client_id:
+        :param request_handler:
+        :return: dict
+        """
+        if client_id and product_id:
+            return request_handler.make_request(
+                'GET',
+                '/loans/template?templateType={}&clientId={}&productId={}'.format(template_type, client_id, product_id)
+            )
+
+        return request_handler.make_request(
+            'GET',
+            '/loans/template?templateType={}&clientId={}'.format(template_type, client_id)
+        )
+
+    @classmethod
+    def apply(cls, request_handler, client_id, product_id, principal, expected_disbursement_date=datetime.now()) \
+            -> 'Loan':
+        """Submit a new loan application
+
+        :param request_handler:
+        :param client_id:
+        :param product_id:
+        :param principal:
+        :param expected_disbursement_date:
+        :return: :class:`fineract.objects.loan.Loan`
+        """
+        template = cls.template(request_handler, client_id, product_id=product_id)
+        payload = {
+            'clientId': client_id,
+            'productId': product_id,
+            'principal': principal,
+            'loanTermFrequency': template['termFrequency'],
+            'loanTermFrequencyType': template['termPeriodFrequencyType']['id'],
+            'loanType': 'individual',
+            'numberOfRepayments': template['numberOfRepayments'],
+            'repaymentEvery': template['repaymentEvery'],
+            'repaymentFrequencyType': template['repaymentFrequencyType']['id'],
+            'interestRatePerPeriod': template['interestRatePerPeriod'],
+            'amortizationType': template['amortizationType']['id'],
+            'interestType': template['interestType']['id'],
+            'interestCalculationPeriodType': template['interestCalculationPeriodType']['id'],
+            'transactionProcessingStrategyId': template['transactionProcessingStrategyId'],
+            'expectedDisbursementDate': expected_disbursement_date,
+            'submittedOnDate': datetime.now(),
+        }
+
+        res = request_handler.make_request(
+            'POST',
+            '/loans',
+            json=payload
+        )
+
+        loan_id = res['loanId']
+        return cls(request_handler,
+                   request_handler.make_request(
+                       'GET',
+                       '/loans/{}'.format(loan_id)
+                   ), False)
+
+    def delete(self) -> bool:
+        """Delete a loan application. Only loans with status of "Submitted and awaiting approval"
+
+        :return: bool
+        """
+        res = self.request_handler.make_request(
+            'DELETE',
+            '/loans/{}'.format(self.id)
+        )
+        return res['loanId'] == self.id
+
+    def approve(self, approved_on_date=datetime.now(), approved_loan_amount=None, expected_disbursement_date=None) \
+            -> bool:
+        """Approve a loan application
+
+        :param approved_on_date:
+        :param approved_loan_amount:
+        :param expected_disbursement_date:
+        :return: bool
+        """
+        payload = {
+            'approvedOnDate': approved_on_date
+        }
+
+        if approved_loan_amount:
+            payload['approvedLoanAmount'] = approved_loan_amount
+
+        if expected_disbursement_date:
+            payload['expectedDisbursementDate'] = expected_disbursement_date
+
+        res = self.request_handler.make_request(
+            'POST',
+            '/loans/{}?command=approve'.format(self.id),
+            json=payload
+        )
+        return res['loanId'] == self.id
+
+    def undo_approval(self, note) -> bool:
+        res = self.request_handler.make_request(
+            'POST',
+            '/loans/{}?command=undoApproval'.format(self.id),
+            json={'note': note}
+        )
+        return res['loanId'] == self.id
+
+    def disburse(self, actual_disbursement_date=datetime.now(), transaction_amount=None, fixed_emi_amount=None) -> bool:
+        """Disburse an approved loan application
+
+        :param actual_disbursement_date:
+        :param transaction_amount:
+        :param fixed_emi_amount:
+        :return: bool
+        """
+        payload = {
+            'actualDisbursementDate': actual_disbursement_date
+        }
+
+        if transaction_amount:
+            payload['transactionAmount'] = transaction_amount
+
+        if fixed_emi_amount:
+            payload['fixedEmiAmount'] = fixed_emi_amount
+
+        res = self.request_handler.make_request(
+            'POST',
+            '/loans/{}?command=disburse'.format(self.id),
+            json=payload
+        )
+        return res['loanId'] == self.id
+
+    # def disburse_to_savings(self, actual_disbursement_date=datetime.now(), transaction_amount=None, fixed_emi_amount=None):
+    #     """Disburse an approved loan application to savings
+    #
+    #     :param actual_disbursement_date:
+    #     :param transaction_amount:
+    #     :param fixed_emi_amount:
+    #     :return: bool
+    #     """
+    #     payload = {
+    #         'actualDisbursementDate': actual_disbursement_date
+    #     }
+    #
+    #     if transaction_amount:
+    #         payload['transactionAmount'] = transaction_amount
+    #
+    #     if fixed_emi_amount:
+    #         payload['fixedEmiAmount'] = fixed_emi_amount
+    #
+    #     res = self.request_handler.make_request(
+    #         'POST',
+    #         '/loans/{}?command=disburseToSavings'.format(self.id),
+    #         json=payload
+    #     )
+    #     return res['loanId'] == self.id
+
+    def undo_disbursal(self, note) -> bool:
+        res = self.request_handler.make_request(
+            'POST',
+            '/loans/{}?command=undoDisbursal'.format(self.id),
+            json={'note': note}
+        )
+        return res['loanId'] == self.id
+
+    def make_repayment(self, amount, transaction_date, **kwargs):
+        """Make a repayment on a loan
+
+        :param amount:
+        :param transaction_date:
+        :param kwargs: Extra args see https://demo.mifos.io/api-docs/apiLive.htm#loans_transactions
+        :return: bool
+        """
+        payload = {
+            'transactionDate': transaction_date,
+            'transactionAmount': amount
+        }
+        payload.update(kwargs)
+        # print(payload)
+        res = self.request_handler.make_request(
+            'POST',
+            '/loans/{}/transactions?command=repayment'.format(self.id),
+            json=payload
+        )
+        return res['loanId'] == self.id
+
 
 class LoanStatus(FineractObject):
 
